local util = require("util")
local mod_gui = require("mod-gui")
local crash_site = require("crash-site")

local created_items = function()
  return
  {
    ["iron-plate"] = 8,
    ["wood"] = 1,
    ["pistol"] = 1,
    ["firearm-magazine"] = 10,
    ["burner-mining-drill"] = 1,
    ["stone-furnace"] = 1
  }
end

local respawn_items = function()
  return
  {
    ["pistol"] = 1,
    ["firearm-magazine"] = 10
  }
end

local ship_items = function()
  return
  {
    ["firearm-magazine"] = 8
  }
end

local debris_items = function()
  return
  {
    ["iron-plate"] = 8
  }
end

local ship_parts = function()
  return crash_site.default_ship_parts()
end

local chart_starting_area = function()
  local r = global.chart_distance or 200
  local force = game.forces.player
  local surface = game.surfaces[1]
  local origin = force.get_spawn_position(surface)
  force.chart(surface, {{origin.x - r, origin.y - r}, {origin.x + r, origin.y + r}})
end
function add_titlebar(gui, caption, close_button_name)
  local titlebar = gui.add{type = "flow"}
  titlebar.drag_target = gui
  titlebar.add{
    type = "label",
    style = "frame_title",
    caption = caption,
    ignored_by_interaction = true,
  }
  local filler = titlebar.add{
    type = "empty-widget",
    style = "draggable_space",
    ignored_by_interaction = true,
  }
  filler.style.height = 24
  filler.style.horizontally_stretchable = true
  titlebar.add{
    type = "sprite-button",
    name = close_button_name,
    style = "frame_action_button",
    sprite = "utility/close_white",
    hovered_sprite = "utility/close_black",
    clicked_sprite = "utility/close_black",
    tooltip = "Close",
  }
end
local update_playerList = function(player)
  if player.gui.screen['hello-window'] then
  player.gui.screen['hello-window']['tabbed-pane']['spectate-tab-flow']['playerList'].clear_items()
  for i, player in pairs(game.players) do
    player.gui.screen['hello-window']['tabbed-pane']['spectate-tab-flow']['playerList'].add_item(player.name)
  end
  end
end
local gui = function(player)
  local ui = player.gui.screen.add{ type="frame", name="hello-window", direction="vertical" }
  ui.force_auto_center()
  add_titlebar(ui, "FreeFactorioServer", "close-frame-button")
  local tabbed_pane = ui.add{type="tabbed-pane", name="tabbed-pane"}
  tabbed_pane.style.vertically_stretchable = true
  tabbed_pane.style.horizontally_stretchable = true
  local tab1 = tabbed_pane.add{type="tab", caption="General", name="general-tab"}
  tab1.style.horizontally_stretchable=true
  local a = tabbed_pane.add{type="flow", direction="vertical", name="general-tab-flow"}
  a.style.horizontally_stretchable=true
  a.add{type="label", caption="Welcome to FreeFactorioServer!"}
  tabbed_pane.add_tab(tab1, a)
  local tab2 = tabbed_pane.add{type="tab", caption="Spectate others!", name="spectate-tab"}
  tab2.style.horizontally_stretchable=true
  local b = tabbed_pane.add{type="flow", direction="vertical", name="spectate-tab-flow"}
  b.style.horizontally_stretchable=true
  local playerList = b.add{type="list-box", name="playerList"}
  b.add{type="label", caption="Camera scale:"}
  local camera_scale = b.add{type="slider", name="camera_scale"}
  camera_scale.style.horizontally_stretchable=true
  camera_scale.set_slider_minimum_maximum(0, 5)
  camera_scale.set_slider_value_step(0.01)
  local camera = b.add{type="camera", name="spectate-camera", position={0,0}}
  camera.style.vertically_stretchable=true
  camera.style.horizontally_stretchable=true
  tabbed_pane.add_tab(tab2, b)
  ui.add{type="label", caption="Window size:"}
  local scale = ui.add{type="slider", name="resize_window", minimum_value=500, maximum_value=1000}
  scale.slider_value = 500
  scale.style.horizontally_stretchable=true
  ui.style.height = scale.slider_value
  ui.style.width = scale.slider_value
  local drag_bottom = ui.add{
      type="flow"
  }
  drag_bottom.drag_target = ui
  local bottom_drag = drag_bottom.add{
      type="empty-widget",
      style="draggable_space",
      ignored_by_interaction=true
  }
  bottom_drag.style.horizontally_stretchable=true
  bottom_drag.style.height = 24
  tabbed_pane.selected_tab_index = 1
  for i, player in pairs(game.players) do
    update_playerList(player)
  end
end

local on_gui_value_changed = function(event)
  if event.element and event.element.valid then
    if event.element.name == "resize_window" then
      event.element.parent.style.width = event.element.slider_value
      event.element.parent.style.height = event.element.slider_value
    end
    if event.element.name == "camera_scale" then
      event.element.parent['spectate-camera'].zoom = event.element.slider_value
    end
  end
end

local on_tick = function(event)
  for i, player in pairs(game.players) do
    if player.gui.screen['hello-window'] and player.gui.screen['hello-window'].valid then
    if player.gui.screen['hello-window']['tabbed-pane']['spectate-tab-flow']['playerList'].selected_index == 0 then

    else
      player.gui.screen['hello-window']['tabbed-pane']['spectate-tab-flow']['spectate-camera'].position = game.players[player.gui.screen['hello-window']['tabbed-pane']['spectate-tab-flow']['playerList'].selected_index].position
    end
    end
  end
end

local on_gui_click = function(event)
   if event.element and event.element.valid then
      if event.element.name == "close-frame-button" then
        event.element.parent.parent.destroy()
      end
   end
end

local on_player_created = function(event)
  local player = game.get_player(event.player_index)
  util.insert_safe(player, global.created_items)
  gui(player)
  if false then

    --This is so that other mods and scripts have a chance to do remote calls before we do things like charting the starting area, creating the crash site, etc.
    global.init_ran = true

    chart_starting_area()

    if not global.disable_crashsite then
      local surface = player.surface
      surface.daytime = 0.7
      crash_site.create_crash_site(surface, {-5,-6}, util.copy(global.crashed_ship_items), util.copy(global.crashed_debris_items), util.copy(global.crashed_ship_parts))
      util.remove_safe(player, global.crashed_ship_items)
      util.remove_safe(player, global.crashed_debris_items)
      player.get_main_inventory().sort_and_merge()
      if player.character then
        player.character.destructible = false
      end
      global.crash_site_cutscene_active = true
      crash_site.create_cutscene(player, {-5, -4})
      return
    end

  end

end

local on_player_respawned = function(event)
  local player = game.get_player(event.player_index)
  util.insert_safe(player, global.respawn_items)
end

local on_cutscene_waypoint_reached = function(event)
  if not global.crash_site_cutscene_active then return end
  if not crash_site.is_crash_site_cutscene(event) then return end

  local player = game.get_player(event.player_index)

  player.exit_cutscene()

  if not global.skip_intro then
    if game.is_multiplayer() then
      player.print({"msg-intro"})
    else
      game.show_message_dialog{text = {"msg-intro"}}
    end
  end
end

local skip_crash_site_cutscene = function(event)
  if not global.crash_site_cutscene_active then return end
  if event.player_index ~= 1 then return end
  local player = game.get_player(event.player_index)
  if player.controller_type == defines.controllers.cutscene then
    player.exit_cutscene()
  end
end

local on_cutscene_cancelled = function(event)
  if not global.crash_site_cutscene_active then return end
  if event.player_index ~= 1 then return end
  global.crash_site_cutscene_active = nil
  local player = game.get_player(event.player_index)
  if player.gui.screen.skip_cutscene_label then
    player.gui.screen.skip_cutscene_label.destroy()
  end
  if player.character then
    player.character.destructible = true
  end
  player.zoom = 1.5
end

local on_player_display_refresh = function(event)
  crash_site.on_player_display_refresh(event)
end

local freeplay_interface =
{
  get_created_items = function()
    return global.created_items
  end,
  set_created_items = function(map)
    global.created_items = map or error("Remote call parameter to freeplay set created items can't be nil.")
  end,
  get_respawn_items = function()
    return global.respawn_items
  end,
  set_respawn_items = function(map)
    global.respawn_items = map or error("Remote call parameter to freeplay set respawn items can't be nil.")
  end,
  set_skip_intro = function(bool)
    global.skip_intro = bool
  end,
  get_skip_intro = function()
    return global.skip_intro
  end,
  set_chart_distance = function(value)
    global.chart_distance = tonumber(value) or error("Remote call parameter to freeplay set chart distance must be a number")
  end,
  get_disable_crashsite = function()
    return global.disable_crashsite
  end,
  set_disable_crashsite = function(bool)
    global.disable_crashsite = bool
  end,
  get_init_ran = function()
    return global.init_ran
  end,
  get_ship_items = function()
    return global.crashed_ship_items
  end,
  set_ship_items = function(map)
    global.crashed_ship_items = map or error("Remote call parameter to freeplay set created items can't be nil.")
  end,
  get_debris_items = function()
    return global.crashed_debris_items
  end,
  set_debris_items = function(map)
    global.crashed_debris_items = map or error("Remote call parameter to freeplay set respawn items can't be nil.")
  end,
  get_ship_parts = function()
    return global.crashed_ship_parts
  end,
  set_ship_parts = function(parts)
    global.crashed_ship_parts = parts or error("Remote call parameter to freeplay set ship parts can't be nil.")
  end
}

if not remote.interfaces["freeplay"] then
  remote.add_interface("freeplay", freeplay_interface)
end

local is_debug = function()
  local surface = game.surfaces.nauvis
  local map_gen_settings = surface.map_gen_settings
  return map_gen_settings.width == 50 and map_gen_settings.height == 50
end

local freeplay = {}

freeplay.events =
{
  [defines.events.on_player_created] = on_player_created,
  [defines.events.on_player_respawned] = on_player_respawned,
  [defines.events.on_cutscene_waypoint_reached] = on_cutscene_waypoint_reached,
  ["crash-site-skip-cutscene"] = skip_crash_site_cutscene,
  [defines.events.on_player_display_resolution_changed] = on_player_display_refresh,
  [defines.events.on_player_display_scale_changed] = on_player_display_refresh,
  [defines.events.on_cutscene_cancelled] = on_cutscene_cancelled,
  [defines.events.on_gui_click] = on_gui_click,
  [defines.events.on_tick] = on_tick,
  [defines.events.on_gui_value_changed] = on_gui_value_changed
}

freeplay.on_configuration_changed = function()
  global.created_items = global.created_items or created_items()
  global.respawn_items = global.respawn_items or respawn_items()
  global.crashed_ship_items = global.crashed_ship_items or ship_items()
  global.crashed_debris_items = global.crashed_debris_items or debris_items()
  global.crashed_ship_parts = global.crashed_ship_parts or ship_parts()

  if not global.init_ran then
    -- migrating old saves.
    global.init_ran = #game.players > 0
  end
end

freeplay.on_init = function()
  global.created_items = created_items()
  global.respawn_items = respawn_items()
  global.crashed_ship_items = ship_items()
  global.crashed_debris_items = debris_items()
  global.crashed_ship_parts = ship_parts()

  if is_debug() then
    global.skip_intro = true
    global.disable_crashsite = true
  end

end

return freeplay